/*
 * Copyright (c) 1999, 2000, 2001, 2002, 2006
 *	Craig Leres
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code
 * distributions retain the above copyright notice and this paragraph
 * in its entirety, and (2) distributions including binary code include
 * the above copyright notice and this paragraph in its entirety in
 * the documentation or other materials provided with the distribution
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static const char rcsid[] =
    "@(#) $Id: util.c,v 1.15 2006/06/15 01:11:48 leres Exp $ (XSE)";
#endif

#include <sys/types.h>

#include <ctype.h>
#include <errno.h>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "cf.h"
#include "webindex.h"
#include "util.h"

const char *
escapename(const char *name)
{
	register char *cp;
	register const char *p;
	register size_t n;
	static char *result = NULL;

	if (result != NULL) {
		free(result);
		result = NULL;
	}
	for (n = 0, p = name; *p != '\0'; ++p)
		if (*p == '\'')
			++n;
	/* We need 3 extra characters per escape */
	n *= 3;

	/* Add space for leading and trailing quotes and EOS */
	result = malloc(strlen(name) + n + 3);
	if (result == NULL) {
		fprintf(stderr, "%s: escapenaem: malloc: %s",
		    prog, strerror(errno));
		exit(1);
	}
	p = name;
	cp = result;
	*cp++ = '\'';
	while (*p != '\0') {
		if (*p == '\'') {
			*cp++ = '\'';
			*cp++ = '\\';
			*cp++ = '\'';
		}
		*cp++ = *p++;
	}
	*cp++ = '\'';
	*cp = '\0';

	return (result);
}

char *
getlistval(register struct list *lp, register const char *name,
    register int isdir)
{
	if (lp != NULL)
		for (; lp->l_key != NULL; ++lp)
			if (matchfileordir(name, lp->l_key, isdir)) {
				lp->l_flags |= L_REFERENCED;
				return (lp->l_val);
			}
	return (NULL);
}

int
matchfileordir(register const char *name1, register const char *name2,
    register int isdir)
{
	register int l1, l2;

	if (!isdir)
		return (strcmp(name1, name2) == 0);

	l1 = strlen(name1);
	if (l1 > 0 && name1[l1 - 1] == '/')
		--l1;
	l2 = strlen(name2);
	if (l2 > 0 && name2[l2 - 1] == '/')
		--l2;
	if (l1 != l2)
		return (0);
	return (strncmp(name1, name2, l1) == 0);
}

void
reportunused(register struct list *lp, register const char *what)
{
	if (lp != NULL)
		for (; lp->l_key != NULL; ++lp)
			if ((lp->l_flags & L_REFERENCED) == 0)
				fprintf(stderr, "%s: warning: %s%s: %s\n",
				    prog, prefix, lp->l_key, what);
}

#ifndef HAVE_SETENV
int
setenv(const char *n, const char *v, int overwrite)
{
	static char e[256];

	(void)snprintf(e, sizeof(e), "%s=%s", n, v);
	return (putenv(e));
}
#endif

const char *
simplewd(void)
{
	register char *cp;
	static char wd[1024];

	if (getcwd(wd, sizeof(wd)) == NULL) {
		fprintf(stderr, "%s: simplewd: getcwd: %s\n",
		    prog, strerror(errno));
		exit(1);
	}
	cp = strrchr(wd, '/');
	if (cp == NULL)
		return (wd);
	return (++cp);
}

int
str2val(register const struct s2v *lp, register const char *suffix)
{
	register const struct s2v *tp;

	for (tp = lp; tp->s != NULL; ++tp)
		if (strcmp(suffix, tp->s) == 0)
			break;

	return (tp->v);
}

int
str2valcase(register const struct s2v *lp, register const char *suffix)
{
	register const struct s2v *tp;

	for (tp = lp; tp->s != NULL; ++tp)
		if (strcasecmp(suffix, tp->s) == 0)
			break;

	return (tp->v);
}

int
suck2dot(register FILE *f, char **pp)
{
	register int n, size, len, i;
	char buf[1024];
	register char *p, *cp;

	n = 0;
	size = 256;
	len = 0;
	p = (char *)malloc(size);
	if (p == NULL) {
		fprintf(stderr, "%s: suck2dot: malloc: %s",
		    prog, strerror(errno));
		exit(1);
	}
	*p = '\0';
	*pp = p;
	while (fgets(buf, sizeof(buf), f) != NULL) {
		++n;
		cp = buf;
		/* Eat leading dot; done if there was only one */
		if (*cp == '.' && *++cp == '\n' && cp[1] == '\0')
			break;

		i = strlen(cp);
		if (len + i + 1 > size) {
			size += i + 256;
			*pp = (char *)realloc(*pp, size);
			if (*pp == NULL) {
				fprintf(stderr, "%s: suck2dot: realloc: %s",
				    prog, strerror(errno));
				exit(1);
			}
			p = *pp + len;
		}
		strcpy(p, cp);
		p += i;
		len += i;
	}
	return (n);
}

int
typ2flag(register const struct v2v *lp, register int wtype)
{
	register const struct v2v *vp;

	for (vp = lp; vp->v1 != 0; ++vp)
		if (vp->v1 == wtype)
			break;
	return (vp->v2);
}
